@@ -534,7 +534,7 @@ typedef struct {
 	/* condstore */
 	uint64_t changedsince;
 	/* qresync */
-	bool vanished;
+	gboolean vanished;
 
 	List_T   bodyfetch;
 } fetch_items;
@@ -562,6 +562,12 @@ typedef struct { // map dbmail_messages
 	uint64_t uid;
 	uint64_t rfcsize;
 	uint64_t seq;
+	// physmessage_id
+	uint64_t phys_id;
+        // expunge flag
+        int expunge;
+        // expunged (pushed to client), can be removed
+        int expunged;
 	int status;
 	char internaldate[IMAP_INTERNALDATE_LEN];
 	int flags[IMAP_NFLAGS];
