@@ -745,8 +745,8 @@ int char2date_str(const char *date, Field_T *frag);
  * db-user accessors
  */
 
-bool db_user_active(uint64_t user_idnr);
-int db_user_set_active(uint64_t user_idnr, bool active);
+gboolean db_user_active(uint64_t user_idnr);
+int db_user_set_active(uint64_t user_idnr, gboolean active);
 
 int db_user_get_security_action(uint64_t user_idnr);
 int db_user_set_security_action(uint64_t user_idnr, long int action);
@@ -777,6 +777,7 @@ char * db_returning(const char *s);
 
 uint64_t db_mailbox_seq_update(uint64_t mailbox_id, uint64_t message_id);
 void db_message_set_seq(uint64_t message_id, uint64_t seq);
+int db_move_message(uint64_t message_id, uint64_t mailbox_id);
 
 int db_rehash_store(void);
 
