@@ -69,11 +69,10 @@ unsigned find_end_of_header(const char *h)
 
 	l  = strlen(h);
 
-	while (h++ && i<=l) {
+	while (h++ && i<l) {
 		i++;
 		c = *h;
-		if (c == '\n' && ((p1 == '\n') || (p1 == '\r' && p2 == '\n'))) {
-			if (l > i)
+		if (c == '\n' && ((p1 == '\n') || (p1 == '\r' && p2 == '\n'))) {			if (l > i)
 				i++;
 			break;
 		}
@@ -337,7 +336,7 @@ static GMimeContentType *find_type(const char *s)
 #define MAX_MIME_DEPTH 64
 #define MAX_MIME_BLEN 128
 
-static bool find_boundary(const char *s, char *boundary)
+static gboolean find_boundary(const char *s, char *boundary)
 {
 	const gchar *param;
 	GMimeContentType *type = find_type(s);
@@ -361,11 +360,11 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
        	ResultSet_T r;
 	char internal_date[SQL_INTERNALDATE_LEN];
 	GMimeContentType *mimetype = NULL;
-	int prevdepth, depth = 0, row = 0;
+	volatile int prevdepth, depth = 0, row = 0;
 	volatile int t = FALSE;
-	gboolean got_boundary = FALSE, prev_boundary = FALSE, is_header = TRUE, prev_header, finalized=FALSE;
-	gboolean prev_is_message = FALSE, is_message = FALSE;
-	String_T m = NULL, n = NULL;
+	volatile gboolean got_boundary = FALSE, prev_boundary = FALSE, is_header = TRUE, prev_header, finalized=FALSE;
+	volatile gboolean prev_is_message = FALSE, is_message = FALSE;
+	volatile String_T m = NULL, n = NULL;
 	const void *blob;
 	Field_T frag;
 
@@ -387,7 +386,7 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 				"FROM %smimeparts p "
 				"JOIN %spartlists l ON p.id = l.part_id "
 				"JOIN %sphysmessage ph ON ph.id = l.physmessage_id "
-				"WHERE l.physmessage_id = ? ORDER BY l.part_key,l.part_order ASC", 
+				"WHERE l.physmessage_id = ? ORDER BY l.part_key, l.part_order ASC, l.part_depth DESC", 
 				frag, p_string_str(n), DBPFX, DBPFX, DBPFX);
 		db_stmt_set_u64(stmt, 1, self->id);
 		r = db_stmt_query(stmt);
@@ -454,9 +453,14 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 			if ((depth > 0) && (blist[depth-1][0]))
 				strncpy(boundary, blist[depth-1], MAX_MIME_BLEN-1);
 
-			if (is_header && (!prev_header || prev_boundary || (prev_header && depth>0 && !prev_is_message))) {
-				dprint("\n--%s\n", boundary);
-				p_string_append_printf(m, "\n--%s\n", boundary);
+			if (is_header){
+				if (prev_header && depth>0 && !prev_is_message) {
+					dprint("--%s\n", boundary);
+					p_string_append_printf(m, "--%s\n", boundary);
+				}else if (!prev_header || prev_boundary) {
+					dprint("\n--%s\n", boundary);
+					p_string_append_printf(m, "\n--%s\n", boundary);
+				}
 			}
 
 			p_string_append_printf(m, "%s", str);
@@ -530,7 +534,7 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
 {
 	const char *boundary;
 	const char *preface = NULL, *postface = NULL;
-	int n, i, c;
+	int n = 0, i, c;
 
 	g_return_val_if_fail(GMIME_IS_OBJECT(object), TRUE);
 
@@ -558,7 +562,7 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
 	if (boundary) {
 		n++;
 		m->part_depth--;
-		m->part_order=n;
+		m->part_order++;
 	}
 
 	if (g_mime_content_type_is_type(GMIME_CONTENT_TYPE(content_type), "multipart", "*") &&
@@ -888,7 +892,7 @@ const char * dbmail_message_get_envelope_recipient(const DbmailMessage *self)
 
 void dbmail_message_set_header(DbmailMessage *self, const char *header, const char *value)
 {
-	g_mime_object_set_header(GMIME_OBJECT(self->content), header, value);
+	g_mime_object_prepend_header(GMIME_OBJECT(self->content), header, value);
 }
 
 const gchar * dbmail_message_get_header(const DbmailMessage *self, const char *header)
@@ -1402,7 +1406,7 @@ static int _header_name_get_id(const DbmailMessage *self, const char *header, ui
 	gchar *case_header, *safe_header, *frag;
 	Connection_T c; ResultSet_T r; PreparedStatement_T s;
 	Field_T config;
-	volatile bool cache_readonly = false;
+	volatile gboolean cache_readonly = false;
 	volatile int t = FALSE;
 
 	// rfc822 headernames are case-insensitive
@@ -1977,7 +1981,7 @@ DbmailMessage * dbmail_message_construct(DbmailMessage *self,
 
 static int get_mailbox_from_filters(DbmailMessage *message, uint64_t useridnr, const char *mailbox, char *into, size_t into_n)
 {
-	int t = FALSE;
+	volatile int t = FALSE;
 	uint64_t anyone = 0;
 	PreparedStatement_T stmt;
 	Connection_T c;
@@ -2185,7 +2189,7 @@ dsn_class_t sort_deliver_to_mailbox(DbmailMessage *message,
 
 	// if the mailbox already holds this message we're done
 	GETCONFIGVALUE("suppress_duplicates", "DELIVERY", val);
-	if (strcasecmp(val,"yes")==0) {
+	if (strcasecmp(val,"yes") == 0) {
 		const char *messageid = dbmail_message_get_header(message, "message-id");
 		if ( messageid && ((db_mailbox_has_message_id(mboxidnr, messageid)) > 0) ) {
 			TRACE(TRACE_INFO, "suppress_duplicate: [%s]", messageid);
