@@ -286,15 +286,15 @@ static void imap_session_reset(ImapSession *session)
 	current = session->state;
 	PUNLOCK(session->lock);
 
-    switch (current) {
-        case CLIENTSTATE_AUTHENTICATED:
-        case CLIENTSTATE_SELECTED:
-            session->ci->timeout.tv_sec = server_conf->timeout; 
-            break;
-        default:
-            session->ci->timeout.tv_sec = server_conf->login_timeout; 
-            break;
-    }
+	switch (current) {
+		case CLIENTSTATE_AUTHENTICATED:
+		case CLIENTSTATE_SELECTED:
+			session->ci->timeout.tv_sec = server_conf->timeout; 
+			break;
+		default:
+			session->ci->timeout.tv_sec = server_conf->login_timeout; 
+			break;
+	}
 
 	TRACE(TRACE_DEBUG,"[%p] state [%d] timeout [%lu]", 
             session, current, session->ci->timeout.tv_sec);
@@ -351,6 +351,12 @@ static void send_greeting(ImapSession *session)
 	dbmail_imap_session_set_state(session, CLIENTSTATE_NON_AUTHENTICATED);
 }
 
+static void disconnect_user(ImapSession *session)
+{
+	imap_session_printf(session, "* BYE [Service unavailable.]\r\n");
+	imap_handle_abort(session);
+}
+
 /*
  * the default timeout callback */
 
@@ -372,7 +378,7 @@ void imap_cb_time(void *arg)
 
 		ci_cork(session->ci);
 		if (! (++session->loop % idle_interval)) {
-			imap_session_printf(session, "* OK\r\n");
+			imap_session_printf(session, "* OK Still here\r\n");
 		}
 		dbmail_imap_session_mailbox_status(session,TRUE);
 		dbmail_imap_session_buff_flush(session);
@@ -444,7 +450,7 @@ static void imap_handle_retry(ImapSession *session)
 static void imap_handle_done(ImapSession *session)
 {
 	/* only do this in the main thread */
-	imap_session_printf(session, "* BYE\r\n");
+	imap_session_printf(session, "* BYE Requested\r\n");
 	imap_session_printf(session, "%s OK LOGOUT completed\r\n", session->tag);
 	imap_session_bailout(session);
 }
@@ -601,6 +607,8 @@ int imap_handle_connection(client_sock *c)
 {
 	ImapSession *session;
 	ClientBase_T *ci;
+	struct rlimit fd_limit;
+	int fd_count;
 
 	ci = client_init(c);
 
@@ -617,7 +625,20 @@ int imap_handle_connection(client_sock *c)
 		Capa_remove(session->capa, "LOGINDISABLED");
 	}
 
-	send_greeting(session);
+	fd_count = get_opened_fd_count();
+	if (fd_count < 0 || getrlimit(RLIMIT_NPROC, &fd_limit) < 0) {
+		TRACE(TRACE_ERR,
+			"[%p] failed to retrieve fd limits, dropping client connection",
+			session);
+		disconnect_user(session);
+	} else if (fd_limit.rlim_cur - fd_count < FREE_DF_THRESHOLD) {
+		TRACE(TRACE_WARNING,
+			"[%p] fd count [%d], fd limit [%d], fd threshold [%d]: dropping client connection",
+			session, fd_count, fd_limit.rlim_cur, FREE_DF_THRESHOLD);
+		disconnect_user(session);
+	} else {
+		send_greeting(session);
+	}
 
 	reset_callbacks(session);
 
