@@ -88,7 +88,7 @@ struct cmd_t {
 #define SESSION_OK_WITH_RESP_CODE(VALUE) \
 	SESSION_OK_COMMON("[%s] ", VALUE)
 
-static void _fetch_update(ImapSession *self, MessageInfo *msginfo, bool showmodseq, bool showflags);
+static void _fetch_update(ImapSession *self, MessageInfo *msginfo, gboolean showmodseq, gboolean showflags);
 
 static int check_state_and_args(ImapSession * self, int minargs, int maxargs, ClientState_T state)
 {
@@ -561,13 +561,13 @@ struct expunged_helper {
 	qresync_args *qresync;
 	uint64_t start_expunged;
 	uint64_t last_expunged;
-	bool prev_expunged;
+	gboolean prev_expunged;
 };
 
 static gboolean _get_expunged(uint64_t *id, gpointer UNUSED value, struct expunged_helper *data)
 {
 	MessageInfo *msg = g_tree_lookup(data->msgs, id);
-	bool expunged;
+	gboolean expunged;
 
 	if (! msg)
 		return TRUE;
@@ -702,18 +702,20 @@ static void _ic_select_enter(dm_thread_data *D)
 		dbmail_imap_session_buff_printf(self, "* OK [HIGHESTMODSEQ %" PRIu64 "] Highest\r\n",
 				MailboxState_getSeq(S));
 	}
-
-	if (MailboxState_getExists(S)) { 
-		/* show msn of first unseen msg (if present) */
-		GTree *uids = MailboxState_getIds(S);
-		GTree *info = MailboxState_getMsginfo(S);
-		uint64_t key = 0, *msn = NULL;
-		g_tree_foreach(info, (GTraverseFunc)mailbox_first_unseen, &key);
-		if ( (key > 0) && (msn = g_tree_lookup(uids, &key))) {
-			dbmail_imap_session_buff_printf(self, "* OK [UNSEEN %" PRIu64 "] first unseen message\r\n", *msn);
+	/* UNSEEN first element*/
+	int command_select_allow_unseen = config_get_value_default_int("command_select_allow_unseen", "IMAP", 1);
+	if(self->command_type == IMAP_COMM_SELECT && command_select_allow_unseen == 1){
+		if (MailboxState_getExists(S)) { 
+			/* show msn of first unseen msg (if present) */
+			GTree *uids = MailboxState_getIds(S);
+			GTree *info = MailboxState_getMsginfo(S);
+			uint64_t key = 0, *msn = NULL;
+			g_tree_foreach(info, (GTraverseFunc)mailbox_first_unseen, &key);
+			if ( (key > 0) && (msn = g_tree_lookup(uids, &key))) {
+				dbmail_imap_session_buff_printf(self, "* OK [UNSEEN %" PRIu64 "] first unseen message\r\n", *msn);
+			}
 		}
 	}
-
 	if (self->command_type == IMAP_COMM_SELECT) {
 		okarg = "READ-WRITE";
 		MailboxState_flush_recent(S);
@@ -785,7 +787,7 @@ static void _ic_enable_enter(dm_thread_data *D)
 
 	while ((capability = self->args[self->args_idx++])) {
 		const char *s = p_string_str(capability);
-		bool changed = false;
+		gboolean changed = false;
 		if (MATCH(s, "CONDSTORE") || MATCH(s, "QRESYNC")) {
 			if (Capa_match(self->capa, s)) {
 				if (MATCH(s, "CONDSTORE")) {
@@ -946,6 +948,7 @@ void _ic_delete_enter(dm_thread_data *D)
 				db_commit_transaction(c);
 			CATCH(SQLException)
 				LOG_SQLERROR;
+				db_rollback_transaction(c);
 				t = DM_EQUERY;
 			FINALLY
 				db_con_close(c);
@@ -2023,7 +2026,7 @@ int _ic_thread(ImapSession *self)
 
 int _dm_imapsession_get_ids(ImapSession *self, const char *set)
 {
-	bool found = FALSE;
+	gboolean found = FALSE;
 	
 	dbmail_mailbox_set_uid(self->mailbox,self->use_uid);
 
@@ -2132,9 +2135,9 @@ int _ic_fetch(ImapSession *self)
  * alter message-associated data in selected mailbox
  */
 
-void _fetch_update(ImapSession *self, MessageInfo *msginfo, bool showmodseq, bool showflags)
+void _fetch_update(ImapSession *self, MessageInfo *msginfo, gboolean showmodseq, gboolean showflags)
 {
-	bool needspace = false;
+	gboolean needspace = false;
 
 	uint64_t *msn = g_tree_lookup(MailboxState_getIds(self->mailbox->mbstate), &msginfo->uid);
 
@@ -2220,8 +2223,8 @@ static gboolean _do_store(uint64_t *id, gpointer UNUSED value, dm_thread_data *D
 
 	// reporting callback
 	if ((! cmd->silent) || changed > 0) {
-		bool showmodseq = (changed && (cmd->unchangedsince || self->mailbox->condstore));
-		bool showflags = (! cmd->silent);
+		gboolean showmodseq = (changed && (cmd->unchangedsince || self->mailbox->condstore));
+		gboolean showflags = (! cmd->silent);
 		_fetch_update(self, msginfo, showmodseq, showflags);
 	}
 
@@ -2235,7 +2238,7 @@ static void _ic_store_enter(dm_thread_data *D)
 	struct cmd_t cmd;
 	gboolean update = FALSE;
 	const char *token = NULL;
-	bool needflags = false;
+	gboolean needflags = false;
 	int startflags = 0, endflags = 0;
 	String_T buffer = NULL;
 
@@ -2404,7 +2407,9 @@ static void _ic_store_enter(dm_thread_data *D)
 	if (self->ids_list) {
 		GString *failed_ids = g_list_join_u64(self->ids_list, ",");
 		buffer = p_string_new(self->pool, "");
-		p_string_printf(buffer, "MODIFIED [%s]", failed_ids->str);
+		//according to RFC7162 section 3.1.3.0 MODIFIED keyword should be used as respnse like 
+		//"... OK [MODIFIED 7,9] ..." not "...OK [MODIFIED [7,9]]..."
+		p_string_printf(buffer, "MODIFIED %s", failed_ids->str);
 		g_string_free(failed_ids, TRUE);
 		g_list_free(g_list_first(self->ids_list));
 		self->ids_list = NULL;
@@ -2436,14 +2441,23 @@ static gboolean _do_copy(uint64_t *id, gpointer UNUSED value, ImapSession *self)
 	uint64_t newid = 0;
 	int result;
 	uint64_t *new_ids_element = NULL;
-
+	if (!g_tree_lookup(self->mailbox->mbstate->msginfo, id)){
+		TRACE(TRACE_WARNING,"Copy message [%ld] failed security issue, trying to copy message that are not in this mailbox",*id);
+		//dbmail_imap_session_buff_printf(self, "%s NO security issue, trying to copy message that are not in this mailbox\r\n",self->tag);
+		return FALSE;
+	}
 	result = db_copymsg(*id, cmd->mailbox_id, self->userid, &newid, TRUE);
 	db_message_set_seq(*id, cmd->seq);
 	if (result == -1) {
-		dbmail_imap_session_buff_printf(self, "* BYE internal dbase error\r\n");
-		return TRUE;
+		/* uid not found, according to RFC 3501 section 6.4.8, should continue  */
+		TRACE(TRACE_WARNING,"Copy message [%ld] failed due to missing in database. continue.",*id);
+		/* continue operation, do not close or send various info on connection */
+		//dbmail_imap_session_buff_printf(self, "* BYE internal dbase error\r\n");
+		//return TRUE;
+		return FALSE;
 	}
 	if (result == -2) {
+		TRACE(TRACE_WARNING,"Copy message [%ld] failed due to `%s NO quotum would exceed`",*id,self->tag);
 		dbmail_imap_session_buff_printf(self, "%s NO quotum would exceed\r\n", self->tag);
 		return TRUE;
 	}
