@@ -282,6 +282,8 @@ static int server_start_cli(ServerConfig_T *conf)
 		TRACE(TRACE_ERR, "could not connect to authentication");
 		return -1;
 	}
+	// Disconnect this connection as threads will create their own
+	auth_disconnect();
 
 	srand((int) ((int) time(NULL) + (int) getpid()));
 
@@ -320,6 +322,9 @@ int StartCliServer(ServerConfig_T * conf)
 {
 	assert(conf);
 	server_start_cli(conf);
+#ifdef HAVE_SYSTEMD
+	sd_notify(0, "STOPPING=1");
+#endif
 	return 0;
 }
 
@@ -656,9 +661,6 @@ void server_sig_cb(int UNUSED fd, short UNUSED event, void *arg)
 			mainReload = 1;
 		case SIGPIPE: // ignore
 		break;
-		case SIGUSR1:
-			g_mem_profile();
-		break;
 		default:
 			exit(0);
 		break;
@@ -847,7 +849,9 @@ int server_run(ServerConfig_T *conf)
 
 	if (MATCH(conf->service_name, "IMAP"))
 		dm_queue_heartbeat();
-
+#ifdef HAVE_SYSTEMD
+	sd_notify(0, "READY=1");
+#endif
 	TRACE(TRACE_DEBUG,"dispatching event loop...");
 
 	event_base_dispatch(evbase);
